@@ -1,11 +1,18 @@
 #' Mean from collection of spectra
 #'
-#' A method to compute the mean of values across members of a collections of
-#' spectra. Computes the mean at each wavelength across all the spectra in the
-#' collection returning a spectral object.
+#' Method to compute the "parallel" mean of values across members of a
+#' collection of spectra or of a spectral object containing multiple spectra in
+#' long form.
 #'
-#' @param x An R object Currently this package defines methods for collections of
-#'    spectral objects.
+#' @details Method specializations compute the mean at each wavelength across a
+#'   group of spectra stored in an object of one of the classes defined in
+#'   package 'photobiolgy'. Trimming of extreme values is
+#'   possible (trimmed mean) and omission of NAs is done separately at each
+#'   wavelength. Interpolation is not applied, so all spectra in \code{x} must
+#'   share the same set of wavelengths. An error is triggered if this condition
+#'   is nor fulfilled.
+#'
+#' @param x An R object.
 #' @param trim	numeric The fraction (0 to 0.5) of observations to be trimmed from
 #'   each end of x before the mean is computed. Values of trim outside that
 #'   range are taken as the nearest endpoint.
@@ -14,24 +21,26 @@
 #' @param ...	Further arguments passed to or from other methods.
 #'
 #' @return If \code{x} is a collection spectral of objects, such as a
-#'   "filter_mspct" object, the returned object is of same class as the
-#'   members of the collection, such as "filter_spct", containing the mean
-#'   spectrum.
+#'   \code{"filter_mspct"} object, the returned object is of same class as the
+#'   members of the collection, such as \code{"filter_spct"}, containing the
+#'   summary spectrum, with variables with names tagged for summaries other
+#'   than mean or median.
 #'
-#' @note Trimming of extreme values and omission of NAs is done separately at
-#'   each wavelength. Interpolation is not applied, so all spectra in \code{x}
-#'   must share the same set of wavelengths.
-#'
-#'   Objects of classes raw_spct and cps_spct can contain data from multiple
-#'   scans. This functions are implemented for these classes only for the case
-#'   when all member spectra contain data for a single scan, or spliced into a
-#'   single column in the case of cps_spct members.
+#' @note Objects of classes \code{raw_spct} and \code{cps_spct} can contain data
+#'   from multiple scans in multiple variables or "columns". The methods accept
+#'   as arguments objects of these classes only if spectra contain data for a
+#'   single spectrometer scan. In the case of \code{cps_spct} objects, a single
+#'   column can also contain data from multiple scans spliced into a single
+#'   variable.
 #'
 #' @seealso See \code{\link[base]{mean}} for the \code{mean()} method used for
 #'   the computations.
 #'
 #' @export
 #'
+#' @examples
+#' s_mean(sun_evening.mspct)
+#'
 s_mean <- function(x, trim, na.rm, ...) UseMethod("s_mean")
 
 #' @describeIn s_mean
@@ -43,6 +52,18 @@ s_mean.default <- function(x, trim = 0, na.rm = FALSE, ...) {
   ifelse(is.any_mspct(x), do.call(class(x[[1]])[1], args = list()), NA)
 }
 
+#' @describeIn s_mean
+#'
+#' @export
+#'
+s_mean.generic_spct <- function(x, trim = 0, na.rm = FALSE, ...) {
+  if (getMultipleWl(x) > 1) {
+    s_mean(subset2mspct(x), trim = trim, na.rm = na.rm, ...)
+  } else {
+    x
+  }
+}
+
 #' @describeIn s_mean
 #'
 #' @export
